"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
;
class ConversationBase {
    constructor() {
        this._index = {};
        this.index_by_date = {};
    }
    register(msg) {
        this._index[msg.id] = msg;
        if (!msg.createdAtDate) {
            msg.createdAtDate = new Date(msg.createdAt);
        }
        const [day, month, year] = [
            msg.createdAtDate.getDate(),
            msg.createdAtDate.getMonth() + 1,
            msg.createdAtDate.getFullYear()
        ];
        if (!this.index_by_date[year]) {
            this.index_by_date[year] = {};
        }
        if (!this.index_by_date[year][month]) {
            this.index_by_date[year][month] = {};
        }
        if (!this.index_by_date[year][month][day]) {
            this.index_by_date[year][month][day] = {};
        }
        this.index_by_date[year][month][day][msg.id] = msg;
    }
    unregisterAll() {
        this._index = {},
            this.index_by_date = {};
    }
    /** Get direct messages from a specific month. */
    month(month, year) {
        if (year in this.index_by_date && month in this.index_by_date[year]) {
            const messages = [].concat(...Object.values(this.index_by_date[year][month]).map(e => Object.values(e)));
            return new SubConversation(messages, this.info.me);
        }
        return new SubConversation([], this.info.me);
    }
    /** Find direct messages that matches a query on their text. */
    find(query) {
        return new SubConversation(this.all.filter(e => query.test(e.text)), this.info.me);
    }
    /** Find context around a single direct message. */
    around(id, context = 20) {
        if (id in this._index) {
            const before = [];
            const after = [];
            const current = this._index[id];
            // Récupération des msg le précédant
            let current_observed_msg = current.previous;
            for (let i = 0; i < context && current_observed_msg !== null; i++) {
                before.push(current_observed_msg);
                current_observed_msg = current_observed_msg.previous;
                i++;
            }
            // Flip in order to have the older messages being on top.
            before.reverse();
            // Récupération des messages le suivant
            current_observed_msg = current.next;
            for (let i = 0; i < context && current_observed_msg !== null; i++) {
                after.push(current_observed_msg);
                current_observed_msg = current_observed_msg.next;
                i++;
            }
            return {
                before,
                current,
                after
            };
        }
        return undefined;
    }
    sender(ids) {
        if (typeof ids === 'string') {
            ids = new Set([ids]);
        }
        if (Array.isArray(ids)) {
            ids = new Set(ids);
        }
        return new SubConversation(this.all.filter(m => ids.has(m.senderId)), this.info.me);
    }
    recipient(ids) {
        if (typeof ids === 'string') {
            ids = new Set([ids]);
        }
        if (Array.isArray(ids)) {
            ids = new Set(ids);
        }
        return new SubConversation(this.all.filter(m => ids.has(m.recipientId)), this.info.me);
    }
    between(since, until) {
        if (typeof since === 'string' && typeof until === 'string') {
            return this.betweenIds(since, until);
        }
        if (since instanceof Date && until instanceof Date) {
            until.setDate(until.getDate() + 1);
            const since_time = since.getTime();
            const until_time = until.getTime();
            const valids = [];
            for (const dm of this) {
                if (dm.createdAtDate.getTime() >= since_time && dm.createdAtDate.getTime() <= until_time) {
                    valids.push(dm);
                }
            }
            return new SubConversation(valids, this.info.me);
        }
        throw new Error("since and until must both be strings, or both be Date objects.");
    }
    betweenIds(id1, id2) {
        if (BigInt(id1) > BigInt(id2)) {
            const tmp = id2;
            id2 = id1;
            id1 = tmp;
        }
        let current = this.single(id1);
        const return_value = [];
        while (current !== null && current.id !== id2) {
            return_value.push(current);
            current = current.next;
        }
        return new SubConversation(return_value, this.info.me);
    }
    /** Retreives a single message (undefined if message does not exists). */
    single(id) {
        return this._index[id];
    }
    /** Iterates all over the direct messages stored in this conversation. */
    *[Symbol.iterator]() {
        yield* this.all;
    }
    /** Participants IDs */
    get participants() {
        return this.info.participants;
    }
    /**
     * Participants IDs, but without yourself.
     * If <return_value>.size > 1, conversation is a group conversation
     */
    get real_participants() {
        const tmp = new Set(this.info.participants);
        tmp.delete(this.info.me);
        return tmp;
    }
    /** True if this conversation is a group conversation. */
    get is_group_conversation() {
        return this.info.participants.size > 2;
    }
    /** All the messages in this conversation */
    get all() {
        return Object.values(this._index);
    }
    /** Count of total messages, messages per year, month, day, plus the messages themselves. */
    get index() {
        // Total count
        const info = {
            count: 0,
            years: {}
        };
        // Count for each year separatly
        for (const [year, y_msgs] of Object.entries(this.index_by_date)) {
            info.years[year] = {
                count: 0,
                months: {}
            };
            // Count for each month separatly
            for (const [month, m_msgs] of Object.entries(y_msgs)) {
                info.years[year].months[month] = {
                    count: 0,
                    days: {}
                };
                // Count for each day separatly
                for (const [day, d_msgs] of Object.entries(m_msgs)) {
                    const m = Object.values(d_msgs);
                    info.years[year].months[month].days[day] = {
                        count: m.length,
                        messages: m
                    };
                    // Mise à jour compteurs intermédiaires
                    info.count += m.length;
                    info.years[year].count += m.length;
                    info.years[year].months[month].count += m.length;
                }
            }
        }
        return info;
    }
    /** Messages sorted by year, month and day, without any other informations */
    get raw_index() {
        return this.index_by_date;
    }
    /** Number of messages in this conversation */
    get length() {
        return this.all.length;
    }
}
/**
 * Conversation between the owner of archive and one or more participants.
 */
class Conversation extends ConversationBase {
    /**
     * Create a new Conversation instance, from raw GDPR conversation.
     * Need self user_id to recognize which user is you.
     *
     * Note that **.add()** is automatically call with the given conversation.
     */
    constructor(conv, me_id) {
        super();
        this.unindexed = [];
        this.info = { id: conv.dmConversation.conversationId, me: me_id, participants: new Set };
        this.add(conv);
    }
    /**
     * Add a new conversation part to actual conversation.
     * Actual conversation and new part must have the same ID.
     *
     * After you've imported all parts, you **must** call **.indexate()** to see messages !
     */
    add(conv) {
        // TODO optimize
        if (conv.dmConversation.conversationId !== this.info.id) {
            throw new Error("You must add into a existing conversation a conversation with the same ID");
        }
        this.unindexed.push(...conv.dmConversation.messages
            .map(e => e.welcomeMessageCreate ? e.welcomeMessageCreate : e.messageCreate)
            .filter(e => e) // Supprime les possibles undefined (clé messageXXX non connue)
        );
    }
    /**
     * Index imported messages.
     * Needed to see all DMs.
     *
     * Should be call after you've imported all with **.add()**.
     */
    indexate() {
        const participants = new Set();
        // Récupération des messages et tri par le plus vieux (ID le plus bas)
        const msgs = this.unindexed
            .concat(this.all) // Ajoute le set de messages actuel (réindexe tout)
            .sort((a, b) => Number(BigInt(a.id) - BigInt(b.id)));
        this.unindexed = [];
        this.unregisterAll();
        // Indexation (ajout d'une clé next et previous)
        let previous_message = null;
        for (const actual_msg of msgs) {
            const swallow = actual_msg;
            if (!swallow.recipientId || !swallow.senderId)
                continue;
            swallow.previous = previous_message;
            swallow.next = null;
            swallow.createdAtDate = new Date(swallow.createdAt);
            if (previous_message) {
                previous_message.next = swallow;
            }
            previous_message = swallow;
            // Enregistrement participants
            if (swallow.recipientId !== "0")
                participants.add(swallow.recipientId);
            participants.add(swallow.senderId);
            // Enregistrement dans l'index
            this.register(swallow);
        }
        // Enregistrement infos
        this.info.participants = participants;
    }
    /** Conversation ID */
    get id() {
        return this.info.id;
    }
}
exports.Conversation = Conversation;
/** Part of a real conversation that have its own index and message count. */
class SubConversation extends ConversationBase {
    constructor(messages, me_id) {
        super();
        const participants = new Set();
        for (const actual_msg of messages) {
            // Enregistrement participants
            if (actual_msg.recipientId !== "0")
                participants.add(actual_msg.recipientId);
            participants.add(actual_msg.senderId);
            // Enregistrement dans l'index
            this.register(actual_msg);
        }
        // Enregistrement infos
        this.info = {
            participants,
            me: me_id
        };
    }
}
exports.SubConversation = SubConversation;
exports.default = Conversation;
